package cn.tonyandmoney.tuon.core.config

import cn.tonyandmoney.tuon.core.cros.GlobalCrosFilter
import cn.tonyandmoney.tuon.core.cros.GlobalWebCrosFilter
import cn.tonyandmoney.tuon.core.properties.CustomConfigProperties
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer
import com.fasterxml.jackson.module.kotlin.registerKotlinModule
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder
import org.springframework.web.server.WebFilter
import reactor.core.publisher.Flux
import java.text.SimpleDateFormat


/**
 * 通用的核心配置
 */
@Configuration
@EnableConfigurationProperties(value = [CustomConfigProperties::class])
class CustomCoreConfiguration {


    @Bean
    @ConditionalOnProperty(prefix = "custom.config.enable", name = ["cros"], havingValue = "true", matchIfMissing = true)
    @ConditionalOnClass(Flux::class)
    fun fluxCrosFilter(): WebFilter {
        return GlobalCrosFilter()
    }

    @Bean
    @ConditionalOnProperty(prefix = "custom.config.enable", name = ["cros"], havingValue = "true", matchIfMissing = true)
    fun crosFilter(): GlobalWebCrosFilter {
        return GlobalWebCrosFilter()
    }

    @Bean("xmlObjectMapper")
    fun xmlObjectMapper(): ObjectMapper {
        return Jackson2ObjectMapperBuilder.xml().build()
    }


    @Bean
    @Primary
    fun builder(): Jackson2ObjectMapperBuilder {
        return Jackson2ObjectMapperBuilder().apply {
            serializerByType(java.lang.Long::class.java, ToStringSerializer.instance)
            serializerByType(java.lang.Long.TYPE, ToStringSerializer.instance)
            // date
            dateFormat(SimpleDateFormat("yyyy-MM-dd HH:mm:ss"))
            timeZone("GMT+8")
        }
    }

    /**
     * 注册Kotlin的Module
     */
    @Bean
    @Primary
    fun objectMapper(): ObjectMapper {
        val mapper: ObjectMapper = builder().build()
        mapper.registerKotlinModule()
        return mapper
    }


}